#pragma once

#include "NPMessageHandler.h"
#include "UpdaterDocument.h"
#include "MVC/Command/NotifyWindowCommand.h"
#include "CheckServiceUpgradeHelper.h"
#include "TimerEventTrigger.h"
#include "MVC/Command/InitCommand.h"
#include <atlpath.h>
#include "trayicon.h"
#include "../../../common/corevalueregistryconst.h"
#include "Registry.h"
#include "../tipupgrader/tipupgraderutil.h"
#include "../../../lib/tipbase/HttpClient/httpclient.h"
#include "TlHelp32.h"

const TCHAR LAUNCHER_GUID[] = _T("EE77F315-BA0A-4ec2-A88F-3A0D6121B831");
const TCHAR LAUNCHERUPGRADER_GUID[] = _T("EB6E12FB-0510-4dfc-A108-3753E54B6174");

const CString TARGET_LAUNCHER("launcher");
const CString TARGET_NTALKBIZ("ntalkbiz"); 
const CString TARGET_UPDATER("updater"); 

typedef CString _String;
typedef std::vector<CString > _StringList;
static LPCTSTR kVerskonToken = _T(".,");

static int _StringTokenize( const _String& text, LPCTSTR szToken, OUT _StringList& outList )
{
	outList.clear();
	if ( text.IsEmpty())
	{
		return 0;
	}

	int pos = 0;
	CAtlString tok = text.Tokenize(szToken, pos);
	while(tok.GetLength())
	{
		outList.push_back( tok );
		tok = text.Tokenize(szToken, pos);
	}
	return (int)outList.size();
}

static BOOL IsVersionUpdatable( LPCTSTR szLocal, LPCTSTR szServer )
{
	CAtlString strLocal = szLocal, strServer = szServer;
	_StringList local, server;

	int localVersionLen = _StringTokenize(strLocal, kVerskonToken, local); 
	int serverVersionLen = _StringTokenize(strServer, kVerskonToken, server); 

	if(localVersionLen == serverVersionLen)
	{
		for(int i = 0; i < serverVersionLen; i++)
		{
			if( _ttoi(server[i]) > _ttoi(local[i]) )
			{
				return TRUE;
			}
		}
	}
	else 
		return TRUE; 

	return FALSE;
}

using namespace tipupgrader;

namespace tipupdater
{
	class CheckUpgradeEventTrigger : public TimerEventTrigger
	{
	public:
		CheckUpgradeEventTrigger(UpdaterDocument *doc, const TCHAR* fileName, int defaultInterval):m_doc(doc), m_defaultInterval(defaultInterval),m_errorCode(0) 
		{
			m_fileName = fileName; 
		}
		~CheckUpgradeEventTrigger() {}

		BOOL Run()
		{
			m_errorCode = 0;
			// Ʈ 
			CheckNetworkCommand* command = dynamic_cast<CheckNetworkCommand*>(TipCommandFactory::get_instance()->CreateAction(_T("CheckNetworkCommand")));
			if (command)
			{
				command->ShowMessageBox(FALSE);
				if (command->Execute() == FALSE)
				{
					m_errorCode = 1;
					return FALSE;
				}
			}

			Application* app = Application::get_instance();
			UpdaterModel* model = dynamic_cast<UpdaterModel*>(app->GetModel());
			UpdaterView* view = dynamic_cast<UpdaterView*>(app->GetView());
			UpdaterDocument* document = dynamic_cast<UpdaterDocument*>(model->GetDocument());
			// Pendingÿ Update ʴ´.
			if (document->GetPendingService())
				return FALSE;

			//UpdateLauncher Ʈ
			UpdateLauncher();

			//Ntalk, Upgrader Ʈ
			TCHAR programPath[MAX_PATH] = {0, };			
			CString strRootPath;
			GetCurrentDirectoryW(MAX_PATH, programPath);			
			strRootPath.AppendFormat(_T("%s\\%s"), programPath, m_fileName);			

			if (view->IsVisibleMainWindow())
				ShellExecute(NULL, NULL, strRootPath, _T("/A"), NULL, SW_SHOWNORMAL);
			else
				ShellExecute(NULL, NULL, strRootPath, _T("/M /A"), NULL, SW_SHOWNORMAL);

			SetInterval(m_defaultInterval);
			return TRUE;
		}

		BOOL SaveVersion(LPCTSTR target)
		{

			CString configFile = m_util.GetConfigFilePath(); 
			TiXmlDocument doc((CStringA)configFile); 
			bool bLoad = doc.LoadFile(); 
			if (bLoad) 
			{ 
				TiXmlElement* pElement = doc.FirstChildElement("root");
				if (!pElement)
					return FALSE; 

				pElement = pElement->FirstChildElement("locale");
				pElement = pElement->FirstChildElement((CStringA)target);
				if (!pElement)
					return FALSE; 

				pElement = pElement->FirstChildElement("version");
				if (!target) 
					return FALSE; 

				TiXmlNode* pChildNode = pElement->FirstChild();
				if (!pChildNode)
					return FALSE; 

				pChildNode->SetValue((CStringA)m_localVersion);
				doc.SaveFile((CStringA)configFile);
			} 
			return TRUE; 
		}

		BOOL LoadServerInfo(LPCTSTR target)
		{
			CString configFile = m_util.GetConfigFilePath(); 

			TCHAR serverfilePath[tipbase::MAXURL] = {0, };
			if (FALSE == ConfigXmlParser::GetURL(target, configFile, Element::ServerInfo, serverfilePath, tipbase::MAXURL, TRUE))
			{
				return FALSE;
			}

			tipbase::GenericHTTPClient httpClient;
			if( httpClient.Request(NULL, serverfilePath) == TRUE)
			{
				TiXmlDocument xmlDoc;
				xmlDoc.Parse((const char*)httpClient.QueryHTTPResponse());
				TiXmlElement* element = xmlDoc.FirstChildElement("root");
				if (!element)
					return FALSE; 

				element = element->FirstChildElement((CStringA)target);
				if (element == NULL)
					return FALSE; 

				TCHAR version[tipbase::MAXSTEXT] = {0, };
				if(FALSE == XMLUtil::GetValue(element, "version", version, tipbase::MAXSTEXT))
				{
					_tcscpy_s(version, tipbase::MAXSTEXT, _T("serverversion loadfail"));
				}

				TCHAR url[tipbase::MAXURL] = {0, };
				if (FALSE == XMLUtil::GetValue(element, "url", url, tipbase::MAXURL))
					return FALSE;

				m_serverVersion = version;
				m_setupFilePath = url;
			}		
			return TRUE;
		}

		BOOL LoadLocalInfo(LPCTSTR target)
		{
			CString configFile = m_util.GetConfigFilePath();

			TCHAR version[tipbase::MAXSTEXT] = {0, };
			if (FALSE == ConfigXmlParser::GetURL(target, configFile, Element::Version, version, tipbase::MAXSTEXT, TRUE))
			{
				return FALSE;
			}

			m_localVersion = version;

			return TRUE;
		}

		CString DownloadUpdateFile()
		{
			tipbase::GenericHTTPClient httpClient;
			TCHAR tempPath[MAX_PATH] = {0, };
			m_util.GetTemporaryPath(tempPath, MAX_PATH);

			CString fileName = ATLPath::FindFileName(m_setupFilePath);
			CString filePath = tempPath;
			filePath += _T("\\");
			filePath += fileName;

			if(httpClient.Request(NULL, m_setupFilePath, GenericHTTPClient::RequestGetMethod, filePath))
				return filePath;

			return CString();
		}

		void MakeParameter(CString& dstParameter, LPCTSTR srcParameter)
		{
			if (srcParameter)
			{
				dstParameter += srcParameter;
			}
			std::vector<CString>::iterator it = m_CmdParameter->begin();
			for (; it != m_CmdParameter->end(); it++)
			{
				dstParameter += _T(" ");
				dstParameter += (*it);
			}
		}

		BOOL RunUpgraderFile(LPCTSTR filepath, LPCTSTR runParameter)
		{
			CString newParameter(_T("/SILENT"));
			HANDLE waitProcessHandle = m_util.ExecuteProgram(filepath, newParameter);

			DWORD ret = WAIT_TIMEOUT;
			MSG msg;
			DWORD ExitCode = 1;
			while(ret == WAIT_TIMEOUT)
			{
				ret = ::WaitForSingleObject(waitProcessHandle, 100);
				if (::PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE) ||
					::PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) 
				{				
					::TranslateMessage(&msg);
					::DispatchMessage(&msg);
				}
				if (ret == WAIT_OBJECT_0)
					GetExitCodeProcess(waitProcessHandle, &ExitCode);
			}
			return (ExitCode == 0);
		}

		BOOL KillNtalkbizLauncherProcess() 
		{
			const TCHAR NTOOLSUPDATER_FILENAME[] = _T("NtalkbizUpgraderLauncher.exe");

			BOOL bResult = TRUE;
			PROCESSENTRY32 entry; 
			entry.dwSize = sizeof(PROCESSENTRY32); 
			CString processName = (CString)NTOOLSUPDATER_FILENAME; 

			HANDLE snapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, NULL); 

			if (Process32First(snapshot, &entry) == TRUE) 
			{ 
				while (Process32Next(snapshot, &entry) == TRUE) 
				{ 
					if (processName.CompareNoCase(entry.szExeFile) == 0) 
					{
						HANDLE hProcess;
						hProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, entry.th32ProcessID);

						if (TerminateProcess(hProcess, 0))
						{
							bResult = TRUE;
						}
						else
						{
							bResult = FALSE;
						}

						break;
					} 
				} 
			} 

			CloseHandle(snapshot);

			return bResult;
		}

		BOOL RunUpgradeLauncher()
		{
			BOOL bSuccess = FALSE;
			HANDLE nsisMutex = NULL;

			if (FALSE == m_util.CreateMutex(LAUNCHERUPGRADER_GUID, nsisMutex))
				return bSuccess;

			//NTalk Launcher 
			if (FALSE == KillNtalkbizLauncherProcess())
			{
				if (nsisMutex)
					m_util.DeleteMutex(nsisMutex);

				return FALSE;
			}

			CString path = DownloadUpdateFile();

			if (path.GetLength() > 0)
			{
				bSuccess = RunUpgraderFile(path, _T(""));
			}

			if (nsisMutex)
				m_util.DeleteMutex(nsisMutex);

			return bSuccess;
		}

		BOOL IsVersionChanged()
		{		
			BOOL bChanged = IsVersionUpdatable(m_localVersion, m_serverVersion);
			if (bChanged == TRUE) 
				m_localVersion = m_serverVersion; 
			return bChanged; 
		}

		BOOL UpdateLauncher() 
		{
			BOOL result = FALSE;
			if (FALSE == m_util.CheckNetwork())
			{
				return FALSE;
			}

			if (FALSE == LoadServerInfo(TARGET_LAUNCHER))
			{
				return FALSE;
			}

			if (FALSE == LoadLocalInfo(TARGET_LAUNCHER))
			{
				return FALSE;
			}

			if (IsVersionChanged())
			{
				if (RunUpgradeLauncher())
					SaveVersion(TARGET_LAUNCHER);


				result = TRUE;
			}

			return result; 

		}

		void Error()
		{
			if (GetInterval() == m_defaultInterval)
			{
				// ǳ 
				if (m_errorCode == 1)
				{
					CTrayIcon* trayIcon = m_doc->GetTrayIcon();
					if (trayIcon)
					{
						trayIcon->PopupBalloon(Application::LoadString(_T("IDS_TOOLTIP_NETWORK_ERROR")), TRUE, 3000);
					}
				}			
			}
			SetInterval(10);
		}

	private:
		UpdaterDocument*	m_doc;
		int					m_defaultInterval;
		int					m_errorCode;
		const TCHAR*		m_fileName;
		UpgraderUtil		m_util;
		CString				m_localVersion;
		CString				m_serverVersion;
		CString				m_setupFilePath;
		std::vector<CString>* m_CmdParameter;
	};

};